/*
 * Copyright 2012 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Eclipse Public License version 1.0, available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.jboss.aesh.console;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;

import org.junit.Test;

/**
 * @author <a href="mailto:stale.pedersen@jboss.org">Ståle W. Pedersen</a>
 */
public class ConsoleTest extends BaseConsoleTest {


    @Test
    public void multiLine() throws IOException, InterruptedException {
        PipedOutputStream outputStream = new PipedOutputStream();
        PipedInputStream pipedInputStream = new PipedInputStream(outputStream);

        Console console = getTestConsole(pipedInputStream);
        console.setConsoleCallback(new AeshConsoleCallback() {
            @Override
            public int execute(ConsoleOperation output) {
                assertEquals("ls foo bar", output.getBuffer());
                return 0;
            }
        });
        console.start();

        outputStream.write(("ls \\").getBytes());
        outputStream.write(("\n").getBytes());
        outputStream.write(("foo \\").getBytes());
        outputStream.write(("\n").getBytes());
        outputStream.write(("bar\n").getBytes());
        outputStream.flush();

        Thread.sleep(100);
        console.stop();
    }


    @Test
    public void testPrintWriter() throws IOException, InterruptedException {
        PipedOutputStream outputStream = new PipedOutputStream();
        PipedInputStream pipedInputStream = new PipedInputStream(outputStream);

        Console console = getTestConsole(pipedInputStream);
        console.setConsoleCallback(new AeshConsoleCallback() {
            @Override
            public int execute(ConsoleOperation output) {
                assertEquals("ls foo bar", output.getBuffer());
                return 0;
            }
        });
        console.start();

        PrintStream out = console.getShell().out();

        out.print("ls \\");
        out.print("\n");
        out.print("foo \\");
        out.print("\n");
        out.println("bar");
        outputStream.flush();

        Thread.sleep(100);
        console.stop();
    }

}
